%%
%
% This code is 99.9% based on "Distributed Gibbs Sampling of Latent Topic
% Models" and corresponding code. 
% If something's wrong with this, please let me know
% : sungjoon.s.choi@gmail.com
%

% Okay, lets start. First lets initialize
clc; clear; close all;

% LDA was first proposed to handle documents
% However, it is hard to 'visualize' words and topics, we will use some
% basis patches in 2D as topics.
% Generate documents
plot_fig = 0;
save_png = 0;
show_text = 0;
gen_documents;
% save('document.mat', 'x', 'K', 'word_hist', 'topics', 'Gj');
fprintf('Generating documents done. \n');

%% 2. Gibbs sample for LDA (takes a lot of time..)
clc; clear; close all;
load 'document.mat'

% Gibbs sample to learn latent dirichlet allocation
alpha   = 5;   
beta    = 1;
K       = 8;
max_iter        = 300;
burnin_iter     = max_iter/10;
sample_lag      = 20;
plot_flag       = 1;
word_reshape    = [4 4];
print_flag      = 1;
detail_print_flag = 0;
[DictionaryTopic, TopicDocument, log_lik, nr_topic_changed_list] ... 
    = Gibbs_LDA(x, K, alpha, beta, max_iter, burnin_iter, sample_lag ...
    , plot_flag, word_reshape, print_flag, detail_print_flag);

% Save current workspace
% save('lda_result.mat', 'K', 'DictionaryTopic', 'TopicDocument', 'nr_topic_changed_list');

%% 3. Plot final result
clc; clear; close all;
load('document.mat');
load('lda_result.mat');
save_png = 1;
word_reshape    = [4 4];

% 1. Words
% fig_word = figure(1); set(fig_word, 'Position', [200 200 500 400]); colormap 'gray'
% imagesc(reshape(zeros(16,1), [4 4])); axis equal; axis tight;
% title('Bag of Words - Each sqaure represents a word', 'FontSize', 20);
% grid on;
% set(gca, 'xtick', [-0.5:1:4]);
% set(gca, 'ytick', [-0.5:1:4]);
% drawnow;
% if save_png
%     print('-dpng', 'pics/words.png');
% end

% 2. Plot ground truth topics
fig_word = figure(2); set(fig_word, 'Position', [300 200 500 400]);
colormap 'gray'
for i=1:8
    subplot(2,4,i); imagesc(reshape(topics(:,i), [4 4])); axis equal; axis tight; axis off;
    title(['Topic \theta_' num2str(i)])
end;
drawnow;
if save_png
    print('-dpng', 'pics/ref_topic.png');
end

% 3. Plot corpus (12 documents)
fig_corpus = figure(3); set(fig_corpus, 'Position', [400 200 500 400]);
colormap 'winter'
plotw = 2;
ploth = 4;
for i = 1:plotw*ploth
    subplot(plotw, ploth, i);
    imagesc(word_hist(:,:,i)); axis equal; axis tight; axis off;
    % colorbar
    title(['Words of ' num2str(i) '-th doc']);
end;
drawnow;
if save_png
    print('-dpng', 'pics/documents.png');
end

% 4. Plot result of Gibbs
fig_lda = figure(4); set(fig_lda, 'Position', [500 200 500 400]);
colormap 'gray'
for k=1:K
    subplot(2,K/2,k);
    imagesc(reshape(DictionaryTopic(:,k), word_reshape)); axis equal; axis tight; axis off;
    title(['Topic \theta_' num2str(k)])
end
if save_png
    print('-dpng', 'pics/lda_topic.png');
end

% 5. Plot result of LDA: TopicDocument
fig_topic_doc = figure(5); set(fig_topic_doc, 'Position', [600 200 500 400]);
colormap 'gray'
for k=1:K
    subplot(2,K/2,k);
    hold on;
    LDA_topic = TopicDocument(:, k);
    plot(LDA_topic, 'r'); % LDA result 
    plot(Gj(k, :), 'b'); % real
    hold off;
    axis tight; 
    title(['Topic \theta_' num2str(k)])
end
if save_png
    print('-dpng', 'pics/lda_topic_document.png');
end

% 6. Plot result of LDA: Total Topic Change
I = 100;
J = size(TopicDocument, 2);
fig_topic_change = figure(6); set(fig_topic_change, 'Position', [700 200 500 400]);
plot(nr_topic_changed_list/I/J);
xlabel('Iteration');
ylabel('Number of Topic changes');
if save_png
    print('-dpng', 'pics/lda_topic_change.png');
end

% show the history of log likelihood, log(w|z), solid red dots
% corresponding to the iterations when the figures
% figure(2); clf;
% plot(logPw_z,'b-');
% hold on;
% plot(sampling_lag:sampling_lag:max_iter, ...
%     logPw_z(sampling_lag:sampling_lag:max_iter),'ro',...
%     'MarkerSize', 10, 'MarkerFaceColor', 'r');
% xlabel('iterations'); ylabel('log(P(w|z))');
% print('-dpng', 'logPw_z.png');

%%
close all;

